@@ -28,7 +28,7 @@
 //!				.enumerate()
 //!				.map(|(i, v)| (i as u8, v.to_string()))
 //!		);
-//!		store.write_infallible(stream).await.unwrap();
+//!		store.write_infallible(stream, 0).await.unwrap();
 //!
 //!		let result = store.get(2).await.unwrap();
 //!		assert_eq!(result, "two");
@@ -55,7 +55,7 @@
 //!			(75, "seventy-five".to_string()),
 //!			(20, "twenty".to_string())
 //!		]);
-//!		store.write_infallible(stream).await.unwrap();
+//!		store.write_infallible(stream, 4).await.unwrap();
 //!
 //!		let result = store.get(2).await;
 //!		assert!(matches!(result, Err(Error::NotFound(2))));
@@ -375,8 +375,8 @@ where
 	//   * Save current file position (as index_start_pos) and number of entries to the header
 	//   * Write the sorted index down to the file
 	//   * Seek to the beginning of the file and write down the header, with completed_writing = true
-	#[instrument(err, level = "info", skip(self, input_stream, writer), fields(indexkv.path = %self.path.display()))]
-	async fn write_to<S, W>(&mut self, mut input_stream: S, writer: &mut W) -> Result<(), StreamError<S::Error, K>> /* {{{ */
+	#[instrument(err, level = "info", skip(self, input_stream, writer, index_size_hint), fields(indexkv.path = %self.path.display()))]
+	async fn write_to<S, W>(&mut self, mut input_stream: S, writer: &mut W, index_size_hint: usize) -> Result<(), StreamError<S::Error, K>> /* {{{ */
 	where
 		S: TryStream<Ok=(K, T)> + Unpin,
 		W: AsyncWrite + AsyncSeek + Unpin,
@@ -393,7 +393,7 @@ where
 		writer.flush().await?;
 
 		let mut pos = self.data_start_position();
-		let mut index = Vec::new();
+		let mut index = Vec::with_capacity(index_size_hint as usize);
 
 		let mut size = 0u32;
 		let size_of_size = encoder.serialized_size(&size)? as u32;
@@ -439,6 +439,9 @@ where
 	/// The Error type is usually inferrable.  A recommended pattern is to have it be your crate's
 	/// Error type, or [anyhow::Error](https://docs.rs/anyhow/1/anyhow/struct.Error.html).
 	///
+	/// `index_size_hint` is used to preallocate the in-memory index that is eventually written to
+	/// disk, which is strictly a performance optimization.  If it's unknown, passing a 0 is fine.
+	///
 	/// Example:
 	/// ```
 	/// use std::io;
@@ -475,7 +478,7 @@ where
 	///					Err(e) => Err(e)
 	///				})
 	///		).err_into::<MyError>();
-	///		store.write(stream).await?;
+	///		store.write(stream, 0).await?;
 	///		Ok(())
 	/// }
 	///
@@ -489,24 +492,24 @@ where
 	///	}
 	/// ```
 	#[inline]
-	pub async fn write<S>(&mut self, input_stream: S) -> Result<(), StreamError<S::Error, K>> /* {{{ */
+	pub async fn write<S>(&mut self, input_stream: S, index_size_hint: usize) -> Result<(), StreamError<S::Error, K>> /* {{{ */
 	where
 		S: TryStream<Ok=(K, T)> + Unpin,
 		S::Error: std::error::Error
 	{
 		self.index_cache.clear();
 		let mut writer = BufWriter::new(self.file.get_ref().try_clone().await?);
-		self.write_to(input_stream, &mut writer).await
+		self.write_to(input_stream, &mut writer, index_size_hint).await
 	} // }}}
 
 	/// For cases where the incoming data stream is infallible, this allows you to avoid that
 	/// portion of error handling.
 	#[inline]
-	pub async fn write_infallible<S>(&mut self, input_stream: S) -> Result<(), Error<K>> /* {{{ */
+	pub async fn write_infallible<S>(&mut self, input_stream: S, index_size_hint: usize) -> Result<(), Error<K>> /* {{{ */
 	where
 		S: Stream<Item=(K, T)> + Unpin
 	{
-		self.write(input_stream.map(Result::<_, Error<K>>::Ok)).await.map_err(|e| match e {
+		self.write(input_stream.map(Result::<_, Error<K>>::Ok), index_size_hint).await.map_err(|e| match e {
 			StreamError::Internal(e) => e,
 			_ => unreachable!()
 		})
@@ -542,7 +545,7 @@ mod tests {
 		let values = vec![42u8];
 		let path = tempfile::NamedTempFile::new().unwrap().into_temp_path();
 		let mut store: Store<u8, u8> = Store::new(path).await.unwrap();
-		store.write_infallible(stream::iter(values.clone().into_iter().enumerate().map(|(i, v)| { println!("{} -> {}", i, v); (i as u8, v) }))).await.unwrap();
+		store.write_infallible(stream::iter(values.clone().into_iter().enumerate().map(|(i, v)| { println!("{} -> {}", i, v); (i as u8, v) })), 0).await.unwrap();
 
 		let result = store.get(0).await.unwrap();
 		assert_eq!(result, 42u8);
@@ -553,7 +556,7 @@ mod tests {
 		let values = vec![11u8, 32u8];
 		let path = tempfile::NamedTempFile::new().unwrap().into_temp_path();
 		let mut store: Store<i16, u8> = Store::new(path).await.unwrap();
-		store.write_infallible(stream::iter(values.clone().into_iter().enumerate().map(|(i, v)| { println!("{} -> {}", i, v); (i as i16, v) }))).await.unwrap();
+		store.write_infallible(stream::iter(values.clone().into_iter().enumerate().map(|(i, v)| { println!("{} -> {}", i, v); (i as i16, v) })), 0).await.unwrap();
 
 		let result = store.get(0).await.unwrap();
 		assert_eq!(result, 11u8);
@@ -566,7 +569,7 @@ mod tests {
 		let values = vec![47u8, 53u8, 128u8];
 		let path = tempfile::NamedTempFile::new().unwrap().into_temp_path();
 		let mut store: Store<u32, u8> = Store::new(path).await.unwrap();
-		store.write_infallible(stream::iter(values.clone().into_iter().enumerate().map(|(i, v)| { println!("{} -> {}", i, v); (i as u32, v) }))).await.unwrap();
+		store.write_infallible(stream::iter(values.clone().into_iter().enumerate().map(|(i, v)| { println!("{} -> {}", i, v); (i as u32, v) })), 0).await.unwrap();
 
 		let result = store.get(0).await.unwrap();
 		assert_eq!(result, 47u8);
@@ -581,7 +584,7 @@ mod tests {
 		let values = vec![42u16];
 		let path = tempfile::NamedTempFile::new().unwrap().into_temp_path();
 		let mut store: Store<i128, u16> = Store::new(path).await.unwrap();
-		store.write_infallible(stream::iter(values.clone().into_iter().enumerate().map(|(i, v)| { println!("{} -> {}", i, v); (i as i128, v) }))).await.unwrap();
+		store.write_infallible(stream::iter(values.clone().into_iter().enumerate().map(|(i, v)| { println!("{} -> {}", i, v); (i as i128, v) })), 0).await.unwrap();
 
 		let result = store.get(0).await.unwrap();
 		assert_eq!(result, 42u16);
@@ -592,7 +595,7 @@ mod tests {
 		let values = vec![11u16, 32u16];
 		let path = tempfile::NamedTempFile::new().unwrap().into_temp_path();
 		let mut store: Store<i32, u16> = Store::new(path).await.unwrap();
-		store.write_infallible(stream::iter(values.clone().into_iter().enumerate().map(|(i, v)| { println!("{} -> {}", i, v); (i as i32, v) }))).await.unwrap();
+		store.write_infallible(stream::iter(values.clone().into_iter().enumerate().map(|(i, v)| { println!("{} -> {}", i, v); (i as i32, v) })), 0).await.unwrap();
 
 		let result = store.get(0).await.unwrap();
 		assert_eq!(result, 11u16);
@@ -607,7 +610,7 @@ mod tests {
 
 		let path = tempfile::NamedTempFile::new().unwrap().into_temp_path();
 		let mut store: Store<u16, FixedWidth> = Store::new(path).await.unwrap();
-		store.write_infallible(stream::iter(values.clone().into_iter().enumerate().map(|(i, v)| (i as u16, v)))).await.unwrap();
+		store.write_infallible(stream::iter(values.clone().into_iter().enumerate().map(|(i, v)| (i as u16, v))), 512).await.unwrap();
 
 		let mut rng = rand::thread_rng();
 		let mut indices = (0..values.len() as u16).collect::<Vec<u16>>();
@@ -646,7 +649,7 @@ mod tests {
 
 		let path = tempfile::NamedTempFile::new().unwrap().into_temp_path();
 		let mut store: Store<i64, Struct> = Store::new(path).await.unwrap();
-		store.write_infallible(stream::iter(values.clone().into_iter().enumerate().map(|(i, v)| (i as i64, v)))).await.unwrap();
+		store.write_infallible(stream::iter(values.clone().into_iter().enumerate().map(|(i, v)| (i as i64, v))), 256).await.unwrap();
 
 		let mut rng = rand::thread_rng();
 		let mut indices = (0..values.len() as i64).collect::<Vec<i64>>();
@@ -686,7 +689,7 @@ mod tests {
 		let path = tempfile::NamedTempFile::new().unwrap().into_temp_path();
 		for _ in 1..4 {
 			let mut store: Store<u128, Struct> = Store::new(&path).await.unwrap();
-			store.write_infallible(stream::iter(values.clone().into_iter().enumerate().map(|(i, v)| (i as u128, v)))).await.unwrap();
+			store.write_infallible(stream::iter(values.clone().into_iter().enumerate().map(|(i, v)| (i as u128, v))), 256).await.unwrap();
 
 			let mut rng = rand::thread_rng();
 			let mut indices = (0..values.len() as u128).collect::<Vec<u128>>();
@@ -706,7 +709,7 @@ mod tests {
 
 		let path = tempfile::NamedTempFile::new().unwrap().keep().unwrap().1;
 		let mut store: Store<i8, FixedWidth> = Store::new(&path).await.unwrap();
-		store.write_infallible(stream::iter(values.clone().into_iter().enumerate().map(|(i, v)| (i as i8, v)))).await.unwrap();
+		store.write_infallible(stream::iter(values.clone().into_iter().enumerate().map(|(i, v)| (i as i8, v))), 0).await.unwrap();
 
 		let mut stream = store.stream_values().await.unwrap().enumerate();
 		while let Some((i, result)) = stream.next().await {
@@ -727,7 +730,7 @@ mod tests {
 
 		let path = tempfile::NamedTempFile::new().unwrap().keep().unwrap().1;
 		let mut store: Store<u64, FixedWidth> = Store::new(&path).await.unwrap();
-		store.write_infallible(stream::iter(values.clone().into_iter().enumerate().map(|(i, v)| (i as u64, v)))).await.unwrap();
+		store.write_infallible(stream::iter(values.clone().into_iter().enumerate().map(|(i, v)| (i as u64, v))), 512).await.unwrap();
 
 		let mut stream = store.stream_values().await.unwrap().enumerate();
 		while let Some((i, result)) = stream.next().await {
@@ -748,7 +751,7 @@ mod tests {
 
 		let path = tempfile::NamedTempFile::new().unwrap().keep().unwrap().1;
 		let mut store: Store<u64, FixedWidth> = Store::new(&path).await.unwrap();
-		store.write_infallible(stream::iter(values.clone().into_iter().enumerate().map(|(i, v)| (i as u64, v)))).await.unwrap();
+		store.write_infallible(stream::iter(values.clone().into_iter().enumerate().map(|(i, v)| (i as u64, v))), 512).await.unwrap();
 		drop(store);
 		for iteration in 1..4 {
 			println!(">>> 1-{}", iteration);
@@ -785,7 +788,7 @@ mod tests {
 
 		let path = tempfile::NamedTempFile::new().unwrap().into_temp_path();
 		let mut store: Store<u64, FixedWidth> = Store::new(path).await.unwrap();
-		store.write_infallible(stream::iter(values.clone().into_iter().enumerate().map(|(i, v)| (i as u64, v)))).await.unwrap();
+		store.write_infallible(stream::iter(values.clone().into_iter().enumerate().map(|(i, v)| (i as u64, v))), 0).await.unwrap();
 
 		let indices = (0..values.len() as u64).collect::<Vec<u64>>();
 		for i in indices.iter() {
@@ -800,7 +803,7 @@ mod tests {
 
 		let path = tempfile::NamedTempFile::new().unwrap().into_temp_path();
 		let mut store: Store<u64, FixedWidth> = Store::new(path).await.unwrap();
-		store.write_infallible(stream::iter(values.clone().into_iter().enumerate().map(|(i, v)| (i as u64, v)))).await.unwrap();
+		store.write_infallible(stream::iter(values.clone().into_iter().enumerate().map(|(i, v)| (i as u64, v))), 0).await.unwrap();
 
 		let mut rng = rand::thread_rng();
 		let mut indices = (0..values.len() as u64).collect::<Vec<u64>>();
@@ -819,7 +822,7 @@ mod tests {
 
 		let path = tempfile::NamedTempFile::new().unwrap().into_temp_path();
 		let mut store: Store<u64, FixedWidth> = Store::new(path).await.unwrap();
-		store.write_infallible(stream::iter(values.clone().into_iter().enumerate().map(|(i, v)| (i as u64, v)))).await.unwrap();
+		store.write_infallible(stream::iter(values.clone().into_iter().enumerate().map(|(i, v)| (i as u64, v))), 13).await.unwrap();
 
 		let mut rng = rand::thread_rng();
 		let mut indices = (0..values.len() as u64).collect::<Vec<u64>>();
@@ -838,7 +841,7 @@ mod tests {
 
 		let path = tempfile::NamedTempFile::new().unwrap().into_temp_path();
 		let mut store: Store<u64, FixedWidth> = Store::new(path).await.unwrap();
-		store.write_infallible(stream::iter(values.clone().into_iter().enumerate().map(|(i, v)| (i as u64, v)))).await.unwrap();
+		store.write_infallible(stream::iter(values.clone().into_iter().enumerate().map(|(i, v)| (i as u64, v))), 0).await.unwrap();
 
 		let mut rng = rand::thread_rng();
 		let mut indices = (0..values.len() as u64).collect::<Vec<u64>>();
@@ -857,7 +860,7 @@ mod tests {
 
 		let path = tempfile::NamedTempFile::new().unwrap().into_temp_path();
 		let mut store: Store<u64, FixedWidth> = Store::new(path).await.unwrap();
-		store.write_infallible(stream::iter(values.clone().into_iter().enumerate().map(|(i, v)| (i as u64, v)))).await.unwrap();
+		store.write_infallible(stream::iter(values.clone().into_iter().enumerate().map(|(i, v)| (i as u64, v))), 47).await.unwrap();
 
 		let mut rng = rand::thread_rng();
 		let mut indices = (0..values.len() as u64).collect::<Vec<u64>>();
@@ -876,7 +879,7 @@ mod tests {
 
 		let path = tempfile::NamedTempFile::new().unwrap().into_temp_path();
 		let mut store: Store<u64, FixedWidth> = Store::new(path).await.unwrap();
-		store.write_infallible(stream::iter(values.clone().into_iter().enumerate().map(|(i, v)| (i as u64, v)))).await.unwrap();
+		store.write_infallible(stream::iter(values.clone().into_iter().enumerate().map(|(i, v)| (i as u64, v))), 0).await.unwrap();
 
 		let mut rng = rand::thread_rng();
 		let mut indices = (0..values.len() as u64).collect::<Vec<u64>>();
@@ -896,7 +899,7 @@ mod tests {
 		let path = tempfile::NamedTempFile::new().unwrap().into_temp_path();
 		for _ in 1..2 {
 			let mut store: Store<u64, FixedWidth> = Store::new(&path).await.unwrap();
-			store.write_infallible(stream::iter(values.clone().into_iter().enumerate().map(|(i, v)| (i as u64, v)))).await.unwrap();
+			store.write_infallible(stream::iter(values.clone().into_iter().enumerate().map(|(i, v)| (i as u64, v))), 53).await.unwrap();
 
 			let mut rng = rand::thread_rng();
 			let mut indices = (0..values.len() as u64).collect::<Vec<u64>>();
@@ -917,7 +920,7 @@ mod tests {
 
 		let path = tempfile::NamedTempFile::new().unwrap().into_temp_path();
 		let mut store: Store<u64, FixedWidth> = Store::new(path).await.unwrap();
-		store.write_infallible(stream::iter(values.clone().into_iter().enumerate().map(|(i, v)| (i as u64, v)))).await.unwrap();
+		store.write_infallible(stream::iter(values.clone().into_iter().enumerate().map(|(i, v)| (i as u64, v))), 512).await.unwrap();
 
 		let mut rng = rand::thread_rng();
 		let indices = (0..values.len() as u64).collect::<Vec<u64>>();
